'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.byActionType = exports.byTruthyCondition = undefined;

var _redux = require('redux');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var insertRules = function insertRules(_ref) {
  var _ref$rules = _ref.rules;
  var rules = _ref$rules === undefined ? [] : _ref$rules;

  var verifiedRules = rules.filter(verifyStructure);

  return function (store) {
    return function (next) {
      return function (action) {
        var getState = store.getState;

        var truthyReactions = rules.filter(byActionType(action.type)).filter(byTruthyCondition({ state: getState(), action: action })).map(function (rule) {
          return rule.reaction;
        });

        if (truthyReactions.length === 0) return next(action);

        return _redux.compose.apply(undefined, _toConsumableArray(truthyReactions))(store)(next)(action);
      };
    };
  };
};

var verifyStructure = function verifyStructure(rule) {
  var type = rule.type;
  var condition = rule.condition;
  var actionTypes = rule.actionTypes;
  var reaction = rule.reaction;

  // @TODO: Should be better ways of doing schema check.

  var result = typeof type === 'string' && Array.isArray(actionTypes) && typeof condition === 'function' && typeof reaction === 'function';

  if (result === false) {
    throw new TypeError('Rule  "' + type + '" miss a property.');
  }

  return result;
};

var byTruthyCondition = exports.byTruthyCondition = function byTruthyCondition(facts) {
  return function (rule) {
    return rule.condition(facts);
  };
};

var byActionType = exports.byActionType = function byActionType(actionType) {
  return function (rule) {
    return rule.actionTypes.some(function (type) {
      return type === actionType;
    });
  };
};

exports.default = insertRules;